'
'*************************************************************************************************************
'* This source code is provided 'as is' and any express or implied warranties, including,                    *
'* but not limited to, the implied warranties of merchantability and fitness for a particular                *
'* purpose are disclaimed.                                                                                   *
'*                                                                                                           *
'* In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,   *
'* special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute   *
'* goods or services; loss of use, data, or profits;  or business interruption) however caused and on any    *
'* theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)   *
'* arising in any way out of the use of this source code, even if advised of the possibility of such damage. *
'*                                                                                                           *
'* Any views and conclusions contained in the source code and any documentation are those of the author's    *
'* and should not be interpreted as representing official policies,                                          *
'* either expressed or implied, of Crownhill Associates Ltd.                                                 *
'*************************************************************************************************************
'
' General purpose USB CDC test program.
'
' Transmits data from several different buffers.
'
' Uses transmit and receive detection using the Carry flag (STATUS.0)
'
' Communicate via the serial terminal set to the com port that the USB connects too.
'
Declare Reminders = Off                                 ' Disable all reminders
    Device = 18F14K50                                   ' Choose a device with on-board full speed USB
    Declare Xtal = 48                                   ' Set the oscillator speed to 48MHz
    Declare Optimiser_Level = 3
    Declare Reminders = Off                             ' Disable all reminders

    Include "CDC_Descriptor.inc"                        ' Include the CDC descriptors and the USB routines

    Dim MyWord As Word                                  ' General purpose variable
    Dim Array1[20] As Byte                              ' Used to hold some output characters
    Dim Out_Buffer As String * 20                       ' Used to hold some output characters
    Dim In_Buffer As String * 20                        ' Used to hold some input characters
    Dim Code_String As Code = " Code Memory Buffer\n\r",0

    Symbol Carry_Flag = STATUS.0                        ' Set if microcontroller does not have control over the DP buffer

'------------------------------------------------------------------------
' The main program loop starts here
Main:
    'OSCTUNEbits_PLLEN = 1                               ' Enable the device's PLL
    Clear                                               ' Clear all RAM before we start
    All_Digital = True                                  ' Set PORTA and PORTD to digital mode
    DelayMS 10                                          ' Wait for things to stabilise

    USBInit                                             ' Initialise USB
    Repeat                                              ' \
        USBPoll                                         '   Wait for the USB interface to become attached
    Until USB_tConnected = 1 Or USB_tConfigured = 1     ' /
    '
    While 1 = 1                                         ' Create an infinite loop
        Repeat                                          ' Wait for USB input
            USBIn 3, In_Buffer, Auto                    ' Poll the USB and Receive some data from endpoint 3
        Until Carry_Flag = 0                            ' Keep looking until data is able to be received

        Clear In_Buffer                                 ' Then clear the buffer for the next time
        
        __USBOut_Buffer = " Send 38 bytes works:\n\r"          
        '
        Repeat
            USBOut 3, __USBOut_Buffer, Auto             
        Until Carry_Flag = 0                            
        
       
        __USBOut_Buffer = "0102030405060708090A0B0C0D0E0F10111213"
        
        '
        Repeat
            USBOut 3, __USBOut_Buffer, Auto             
        Until Carry_Flag = 0                            
        
        __USBOut_Buffer = "\n\r Send 41 bytes output only 39:\n\r"
        '
        Repeat
            USBOut 3, __USBOut_Buffer, Auto             
        Until Carry_Flag = 0                            
        
        __USBOut_Buffer = "0102030405060708090A0B0C0D0E0F101112131415"    
        '
        ' for some reason can send up to 39 bytes
        ' output  is 0102030405060708090A0B0C0D0E0F101112131 last byte is missing
        '
        Repeat
            USBOut 3, __USBOut_Buffer, Auto             
        Until Carry_Flag = 0                            
        
    Wend                                                

'------------------------------------------------------------------------------------------------
' Configure the 18F14K50 for operation at 48MHz with a crystal frequency of 12MHz
'
Config_Start
    CPUDIV = NOCLKDIV    ' No CPU System Clock divide
    USBDIV = Off         ' USB Clock comes directly from the OSC1/OSC2 oscillator block' no divide
    FOSC = HS            ' HS oscillator
    PLLEN = On           ' Oscillator multiplied by 4
    PCLKEN = On          ' Primary clock enabled
    FCMEN = Off          ' Fail-Safe Clock Monitor disabled
    IESO = Off           ' Oscillator Switchover mode disabled
    PWRTEN = On          ' PWRT enabled
    BOREN = On           ' Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 19            ' VBOR set to 1.9 V nominal
    WDTEN = Off          ' WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 128          ' Watchdog Timer Postscale 1:128
    MCLRE = On           ' MCLR pin enabled, RE3 input pin disabled
    HFOFST = Off         ' The system clock is held off until the HFINTOSC is stable.
    STVREN = Off         ' Stack full/underflow will not cause Reset
    LVP = Off            ' Single-Supply ICSP disabled
    BBSIZ = Off          ' 512W boot block size
    XINST = Off          ' Instruction set extension and Indexed Addressing mode disabled
    Debug = Off          ' Background debugger disabled and RA0 and RA1 configured as general purpose I/O pins
    Cp0 = Off            ' Block 0 not code-protected
    CP1 = Off            ' Block 1 not code-protected
    CPB = Off            ' Boot block not code-protected
    CPD = Off            ' Data EEPROM not code-protected
    WRT0 = Off           ' Block 0 not write-protected
    WRT1 = Off           ' Block 1 not write-protected
    WRTB = Off           ' Boot block not write-protected
    WRTC = Off           ' Configuration registers not write-protected
    WRTD = Off           ' Data EEPROM not write-protected
    EBTR0 = Off          ' Block 0 not protected from table reads executed in other blocks
    EBTR1 = Off          ' Block 1 not protected from table reads executed in other blocks
    EBTRB = Off          ' Boot block not protected from table reads executed in other blocks
Config_End

